# Examples 7.4-7.6 of
# Mehttalo, L. and Lappi, J. 2020. Biometry for forestry and environmental data: With examples in R. CRC Press. 
# 
# Author: Lauri Mehttalo
###############################################################################

library(nlme)
library(lmfor)

### Example 7.4
# Reparameterized logistic function
mylogis<-function(x,phi1,phi2){
         ephi1<-exp(phi1)
         ephi2<-exp(phi2)
         ephi1/(1+exp(4-8*x/ephi2))
         }

data(thinning)
thinning$Ythin<-thinning$Year-1986
thinnls1<-nls(ThEff~mylogis(Ythin,phi1,phi2),
		data=thinning,
		start=list(phi1=log(320),phi2=log(5)))
summary(thinnls1)

thinnls2<-nls(ThEff~SSlogis(Ythin,phi1,phi2,phi3),data=thinning)
summary(thinnls2)

windows(width=7,height=3)
par(mfcol=c(1,2),mai=c(0.6,0.6,0.3,0.1),mgp=c(2,0.7,0),cex=0.9)
plot(thinning$Ythin,thinning$ThEff,type="l",
     xlab="Time since thinning, yr",
     ylab=expression("Thinning effect in Ring Basal Area, "*mm^2),
     lty="dashed")
tapu<-seq(-5,20,0.1)
lines(tapu,mylogis(tapu,log(320),log(3)),lty="solid")
lines(tapu,mylogis(tapu,log(320),log(5)),lty="solid")
lines(tapu,mylogis(tapu,log(320),log(7)),lty="solid")
abline(v=0)

plot(thinning$Ythin,thinning$ThEff,type="l",
     xlab="Time since thinning, yr",
     ylab=expression("Thinning effect in Ring Basal Area, "*mm^2),
     lty="dashed")
abline(v=0)
lines(thinning$Ythin,fitted(thinnls1))
lines(thinning$Ythin,fitted(thinnls2),lwd=2)
###


### Example 7.5 

# The Jacobian
mylogisJ<-function(x,phi1,phi2){
	temp<-8*x*exp(-phi2)
	d1<-exp(phi1)/(1+exp(4-temp))
	d2<--8*x*exp(4-temp+phi1-phi2)*(1+exp(4-temp))^{-2}
	cbind(d1,d2)
}

# Modified Gauss-Newton
y<-thinning$ThEf
x<-thinning$Year-1986
phi<-0
phinew<-c(log(320),log(5))
i<-0
while (sum((phi-phinew)^2)>1e-10){
	phi<-phinew
	i<-i+1
	u<-y-mylogis(x,phi[1],phi[2])
	RSS<-RSSnew<-sum(u^2)
	cat("Iteration: ",i,exp(phi),RSS,"\n")
	X<-mylogisJ(x,phi[1],phi[2])
	psi<-coef(lm(u~X-1))
	lambda<-2
	while (RSSnew>=RSS){
		lambda<-lambda/2
		phinew<-phi+lambda*psi
		unew<-y-mylogis(x,phinew[1],phinew[2])
		RSSnew<-sum(unew^2)
	}
}

sigma2<-RSSnew/(23-2)
varbeta<-sigma2*solve(t(X)%*%X)
phinew              # Parameter estimates
sqrt(sigma2)        # Residual standard error
sqrt(diag(varbeta)) # Standard errors of parameter estimates

# Unmodifoed Gauss-Newton
y<-thinning$ThEf
x<-thinning$Year-1986
phi<-0
phinew<-c(log(320),log(5))
i<-0
while (sum((phi-phinew)^2)>1e-10){
	phi<-phinew
	i<-i+1
	X<-mylogisJ(x,phi[1],phi[2])
	u<-y-mylogis(x,phi[1],phi[2])+X%*%phi
	cat("Iteration: ",i,exp(phi),RSS,"\n")
	phinew<-coef(mod<-lm(u~X-1))
}
sigma2<-RSSnew/(23-2)
varbeta<-sigma2*solve(t(X)%*%X)
phinew              # Parameter estimates
sqrt(sigma2)        # Residual standard error
sqrt(diag(varbeta)) # Standard errros of parameter estimates

### Example 7.6
# Refit the model using gnls to allow the use of ACF and comparison with a  GNLS-model
thinnls1<-gnls(ThEff~mylogis(Ythin,phi1,phi2),
               data=thinning,
              start=list(phi1=log(320),phi2=log(5)))
   
windows(width=3.5,height=3)
par(mfcol=c(1,1),mai=c(0.6,0.6,0.1,0.1),mgp=c(2,0.7,0),cex=0.9)
plot(fitted(thinnls1),resid(thinnls1))
mywhiskers(fitted(thinnls1),resid(thinnls1),limits=c(-100,200,400),add=TRUE,se=FALSE)

windows(width=3.5,height=3)
par(mfcol=c(1,1),mai=c(0.6,0.6,0.1,0.1),mgp=c(2,0.7,0),cex=0.9)
plot(ACF(thinnls1),alpha=0.05)


# GNLS with varaince function and AR(1) residuals
thinnls3<-gnls(ThEff~mylogis(Ythin,phi1,phi2),
               data=thinning,
               start=list(phi1=log(320),phi2=log(5)),
               cor=corAR1(),
               weights=varPower())



anova(thinnls1,thinnls3)

summary(thinnls3)

exp(intervals(thinnls1,which="coef")[[1]])
exp(intervals(thinnls3,which="coef")[[1]])
###

